@@ -3,8 +3,12 @@
 # -----------------------------------------------------------------------------
 from netmiko import ConnectHandler
 import paramiko, traceback
+import pandas as pd
 from time import sleep
 from nettoolkit import STR, IO, IP, LOG
+from facts_finder.database import append_to_xl
+from facts_finder import evaluate
+
 # -----------------------------------------------------------------------------
 
 BAD_CONNECTION_MSG = ': BAD CONNECTION DETECTED, TEARED DOWN'
@@ -264,7 +268,7 @@ class COMMAND():
 	"""    	
 
 	# INITIALIZE class vars
-	def __init__(self, conn, cmd, path):
+	def __init__(self, conn, cmd, path, parsed_output):
 		"""[summary]
 
 		Args:
@@ -275,6 +279,7 @@ class COMMAND():
 		self.conn = conn
 		self.cmd = cmd
 		self.path = path
+		self.parsed_output = parsed_output
 		self._commandOP(conn)
 
 
@@ -287,8 +292,9 @@ class COMMAND():
 		Returns:
 			str: file name where output get stored
 		"""    		
-		if cumulative:			
-			self.fname = self.add_to_file(self.commandOP)    # add to file		
+		if cumulative:
+			self.cumulative_filename = self.add_to_file(self.commandOP)    # add to file
+			self.fname = self.cumulative_filename
 			print(self.cmd, ">> ", self.fname)
 		else:
 			self.fname = self.send_to_file(self.commandOP)    # save to file
@@ -313,7 +319,9 @@ class COMMAND():
 
 		op = self.conn.net_connect.send_command(self.cmd, 
 				read_timeout=20, 
-				delay_factor=self.conn.delay_factor)
+				delay_factor=self.conn.delay_factor,
+				use_textfsm=self.parsed_output,
+				)
 
 		# exclude missed ones
 		if any([								
@@ -370,7 +378,16 @@ class Execute_Device():
 		cumulative (bool, optional): True,False,both. Defaults to False.
 	"""    	
 
-	def __init__(self, ip, auth, cmds, path, cumulative=False, forced_login=False):
+	def __init__(self, 
+		ip, 
+		auth, 
+		cmds, 
+		path, 
+		cumulative, 
+		forced_login, 
+		parsed_output,
+		parse_n_clean,
+		):
 		"""initialize execution
 
 		Args:
@@ -384,7 +401,10 @@ class Execute_Device():
 		self.cmds = cmds
 		self.path = path
 		self.cumulative = cumulative
+		self.cumulative_filename = None
 		self.forced_login = forced_login
+		self.parsed_output = parsed_output
+		self.parse_n_clean = parse_n_clean
 		self.delay_factor, self.dev = None, None
 		pinging = self.check_ping(ip)
 		if forced_login or pinging:
@@ -427,7 +447,8 @@ class Execute_Device():
 				un=self.auth['un'], 
 				pw=self.auth['pw'])
 			return self.dev
-		except:
+		except Exception as e:
+			print(f'{ip}: {e}')
 			return None
 
 	def is_connected(self, c):
@@ -459,7 +480,12 @@ class Execute_Device():
 					devtype=self.dev.dtype,
 					) as c:
 			if self.is_connected(c):
-				self.command_capture(c)
+				cc = self.command_capture(c)
+				self.cumulative_filename = cc.cumulative_filename
+				if self.parsed_output: 
+					xl_file = cc.write_facts()
+				if self.parse_n_clean:
+					cc.clean(xl_file, self.cumulative_filename)
 
 	def command_capture(self, c):
 		"""start command captures on connection object
@@ -467,11 +493,14 @@ class Execute_Device():
 		Args:
 			c (conn): connection object
 		"""    		
-		Captures(dtype=self.dev.dtype, 
+		cc = Captures(dtype=self.dev.dtype, 
 			conn=c, 
 			cmds=self.cmds, 
 			path=self.path, 
-			cumulative=self.cumulative)
+			cumulative=self.cumulative,
+			parsed_output=self.parsed_output
+			)
+		return cc
 
 # -----------------------------------------------------------------------------
 # Execution of Show Commands on a single device. 
@@ -485,7 +514,7 @@ class CLP():
 		conn (conn): connection object
 		path (str): path to store the captured output	
 	"""    	
-	def __init__(self, dtype, conn, path):
+	def __init__(self, dtype, conn, path, parsed_output):
 		"""Initialize object
 
 		Args:
@@ -496,6 +525,10 @@ class CLP():
 		self.dtype = dtype
 		self.conn = conn
 		self.path = path
+		self.parsed_output = parsed_output
+		self.cumulative_filename = None
+		self.parsed_cmd_df = {}
+		self.cmd_exec_logs = []
 		self.hn = self.conn.hn
 		self.ip = self.conn.devvar['ip']
 		self.configure(False)						# fixed disable as now
@@ -533,16 +566,47 @@ class CLP():
 		Returns:
 			[type]: [description]
 		"""    		
-		# cmdObj = COMMAND(conn=self.conn, cmd=cmd, path=self.path)
-		try:
+		self.cmd_exec_logs.append({'command':cmd})
+		cmdObj = self._cmd_capture_raw(cmd, cumulative, banner)
+		if cmdObj is not None:
+			self._cmd_capture_parsed(cmd, cumulative, banner)
+		return cmdObj
 
-			cmdObj = COMMAND(conn=self.conn, cmd=cmd, path=self.path)
-			cmdObj.banner = banner
-			file = cmdObj.op_to_file(cumulative=cumulative)
-			return cmdObj
+	def _cmd_capture_raw(self, cmd, cumulative=False, banner=False):
+		try:
+			cmdObj = COMMAND(conn=self.conn, cmd=cmd, path=self.path, parsed_output=False)
 		except:
 			print(f"{self.hn} : Error executing command {cmd}")
+			self.cmd_exec_logs[-1]['raw'] = False
 			return None
+		try:
+			cmdObj.banner = banner		
+			file = cmdObj.op_to_file(cumulative=cumulative)
+			self.cmd_exec_logs[-1]['raw'] = True
+			if self.cumulative_filename is None:
+				self.cumulative_filename = cmdObj.cumulative_filename
+			return cmdObj
+		except:
+			print(f"{self.hn} : Error writing output for command {cmd}\n{cmdObj.output}")
+			print(self.cmd_exec_logs)
+			self.cmd_exec_logs[-1]['raw'] = False
+			return False
+
+	def _cmd_capture_parsed(self, cmd, cumulative=False, banner=False):
+		try:
+			cmdObj_parsed = COMMAND(conn=self.conn, cmd=cmd, path=self.path, parsed_output=True)
+		except:
+			print(f"{self.hn} : Error executing command - Parse Run {cmd}")
+			self.cmd_exec_logs[-1]['parsed'] = False
+			return None
+		try:
+			self.parsed_cmd_df[cmd] = pd.DataFrame(cmdObj_parsed.output)
+			self.cmd_exec_logs[-1]['parsed'] = True
+		except:
+			print(f"{self.hn} : Error parsing output for command {cmd}")
+			print(f"{self.hn} : data facts may not be available for command: {cmd}")
+			self.cmd_exec_logs[-1]['parsed'] = False
+			return False
 
 
 
@@ -558,7 +622,7 @@ class Captures(CLP):
 
 	"""    	
 
-	def __init__(self, dtype, conn, cmds, path, cumulative=False):
+	def __init__(self, dtype, conn, cmds, path, cumulative=False, parsed_output=False):
 		"""Initiate captures
 
 		Args:
@@ -568,10 +632,11 @@ class Captures(CLP):
 			path (str): path to store the captured output
 			cumulative (bool, optional): True/False/both. Defaults to False.
 		"""    		
-		super().__init__(dtype, conn, path)
+		super().__init__(dtype, conn, path, parsed_output)
 		self.cmds = cmds
 		self.op = ''
 		self.cumulative = cumulative
+		self.cumulative_filename = None
 		self.grp_cmd_capture()
 
 
@@ -590,8 +655,42 @@ class Captures(CLP):
 				return None
 			cc = self.cmd_capture(cmd, self.cumulative, banner)
 			# output = cc.commandOP #if cc else f": Error executing command {cmd}"
-			output = cc.output
+			try:
+				output = cc.output
+			except:
+				output = f": Error executing command {cmd}"
 			cmd_line = self.hn + ">" + cmd + "\n"
 			self.op += cmd_line + "\n" + output + "\n\n"
-			# except: pass
 			banner = ""
+
+
+			# break
+
+	def add_exec_logs(self):
+		self.parsed_cmd_df['logs'] = pd.DataFrame(self.cmd_exec_logs)
+
+	def write_facts(self):
+		"""writes commands facts in to excel tab
+		"""
+		try:
+			self.add_exec_logs()
+			xl_file = self.path + self.conn.hn + ".xlsx"
+			print(f"writing facts to excel: {xl_file}", end="\t")
+			append_to_xl(xl_file, self.parsed_cmd_df, overwrite=True)
+			print(f"...success!")
+		except:
+			print(f" ...failed!")
+		return xl_file
+
+
+	def clean(self, xl_file, cumulative_filename):
+		"""cleans excel fact file. removes some unwanted and adds a few more details.  Creates a new Gene file.
+		"""
+		ev = evaluate(
+			capture_log_file=cumulative_filename,
+			capture_file=xl_file,
+			cisco_var_column_mapper_file=None,
+			cisco_int_column_mapper_file=None,
+			juniper_column_mapper_file=None,
+			)
+		return ev
\ No newline at end of file
